"""Objects for importing Garmin data from Garmin Connect downloads and FIT files."""

__author__ = "Tom Goetz"
__copyright__ = "Copyright Tom Goetz"
__license__ = "GPL"


import sys
import logging
import datetime
import enum

import fitfile
from idbutils import JsonFileProcessor, Conversions

from .garmindb import GarminDb, Attributes, Weight, Sleep, SleepEvents, RestingHeartRate, DailySummary
from .fit_data import FitData


logger = logging.getLogger(__file__)
logger.addHandler(logging.StreamHandler(stream=sys.stdout))
root_logger = logging.getLogger()


class GarminWeightData(JsonFileProcessor):
    """Class for importing JSON formatted Garmin Connect weight data into a database."""

    def __init__(self, db_params, input_dir, latest, measurement_system, debug):
        """
        Return an instance of GarminWeightData.

        Parameters:
        ----------
        db_params (object): configuration data for accessing the database
        input_dir (string): directory (full path) to check for weight data files
        latest (Boolean): check for latest files only
        measurement_system (enum): which measurement system to use when importing the files
        debug (Boolean): enable debug logging

        """
        logger.info("Processing weight data")
        super().__init__(r'weight_\d{4}-\d{2}-\d{2}\.json', input_dir=input_dir, latest=latest, debug=debug)
        self.measurement_system = measurement_system
        self.garmin_db = GarminDb(db_params)
        self.conversions = {'startDate': self._parse_date}

    def _process_json(self, json_data):
        weight_list = json_data['dateWeightList']
        if len(weight_list) > 0:
            weight = fitfile.Weight.from_grams(weight_list[0]['weight'])
            point = {
                'day': json_data['startDate'].date(),
                'weight': weight.kgs_or_lbs(self.measurement_system)
            }
            Weight.insert_or_update(self.garmin_db, point)
            return 1
        return 0


class GarminMonitoringFitData(FitData):
    """Class for importing monitoring FIT files into a database."""

    def __init__(self, input_dir, latest, measurement_system, debug):
        """
        Return an instance of GarminMonitoringFitData.

        Parameters:
        ----------
        input_dir (string): directory (full path) to check for monitoring data files
        latest (Boolean): check for latest files only
        measurement_system (enum): which measurement system to use when importing the files
        debug (Boolean): enable debug logging

        """
        super().__init__(input_dir, debug, latest, True, [fitfile.FileType.monitoring_b], measurement_system)


class GarminSettingsFitData(FitData):
    """Class for importing settings FIT files into a database."""

    def __init__(self, input_dir, debug):
        """
        Return an instance of GarminSettingsFitData.

        Parameters:
        ----------
        input_dir (string): directory (full path) to check for settings data files
        debug (Boolean): enable debug logging

        """
        super().__init__(input_dir, debug, fit_types=[fitfile.FileType.settings])


class SleepActivityLevels(enum.Enum):
    """Enum of values used to encode activity levels during sleep on Gamin Connect."""

    deep_sleep = 0.0
    light_sleep = 1.0
    awake = 2.0
    more_awake = 3.0


class RemSleepActivityLevels(enum.Enum):
    """Enum of values used to encode activity levels during sleep (including REM sleep) on Gamin Connect."""

    unmeasurable = -1.0
    deep_sleep = 0.0
    light_sleep = 1.0
    rem_sleep = 2.0
    awake = 3.0


class GarminSleepData(JsonFileProcessor):
    """Class for importing JSON formatted Garmin Connect sleep data into a database."""

    def __init__(self, db_params, input_dir, latest, debug):
        """
        Return an instance of GarminSleepData.

        Parameters:
        ----------
        db_params (object): configuration data for accessing the database
        input_dir (string): directory (full path) to check for sleep data files
        latest (Boolean): check for latest files only
        debug (Boolean): enable debug logging

        """
        logger.info("Processing sleep data")
        super().__init__(r'sleep_\d{4}-\d{2}-\d{2}\.json', input_dir=input_dir, latest=latest, debug=debug)
        self.garmin_db = GarminDb(db_params)
        self.conversions = {
            'calendarDate': self._parse_date,
            'sleepTimeSeconds': fitfile.conversions.secs_to_dt_time,
            'sleepStartTimestampGMT': Conversions.epoch_ms_to_dt,
            'sleepEndTimestampGMT': Conversions.epoch_ms_to_dt,
            'deepSleepSeconds': fitfile.conversions.secs_to_dt_time,
            'lightSleepSeconds': fitfile.conversions.secs_to_dt_time,
            'remSleepSeconds': fitfile.conversions.secs_to_dt_time,
            'awakeSleepSeconds': fitfile.conversions.secs_to_dt_time,
            'startGMT': self._parse_date,
            'endGMT': self._parse_date
        }

    def _process_json(self, json_data):
        daily_sleep = json_data.get('dailySleepDTO')
        if daily_sleep is None:
            return 0
        date = daily_sleep.get('calendarDate')
        if date is None:
            return 0
        day = date.date()
        if json_data.get('remSleepData'):
            root_logger.info("Importing %s with REM data", day)
            sleep_activity_levels = RemSleepActivityLevels
        else:
            root_logger.info("Importing %s without REM data", day)
            sleep_activity_levels = SleepActivityLevels
        day_data = {
            'day': day,
            'start': daily_sleep.get('sleepStartTimestampGMT'),
            'end': daily_sleep.get('sleepEndTimestampGMT'),
            'total_sleep': daily_sleep.get('sleepTimeSeconds'),
            'deep_sleep': daily_sleep.get('deepSleepSeconds'),
            'light_sleep': daily_sleep.get('lightSleepSeconds'),
            'rem_sleep': daily_sleep.get('remSleepSeconds'),
            'awake': daily_sleep.get('awakeSleepSeconds')
        }
        Sleep.insert_or_update(
            self.garmin_db, day_data, ignore_none=True)
        sleep_levels = json_data.get('sleepLevels')
        if sleep_levels is None:
            return 0
        for sleep_level in sleep_levels:
            start = sleep_level['startGMT']
            end = sleep_level['endGMT']
            event = sleep_activity_levels(sleep_level['activityLevel'])
            duration = (datetime.datetime.min + (end - start)).time()
            level_data = {
                'timestamp': start,
                'event': event.name,
                'duration': duration
            }
            SleepEvents.insert_or_update(
                self.garmin_db, level_data, ignore_none=True)
        return len(sleep_levels)


class GarminRhrData(JsonFileProcessor):
    """Class for importing JSON formatted Garmin Connect resting heart rate data into a database."""

    def __init__(self, db_params, input_dir, latest, debug):
        """
        Return an instance of GarminRhrData.

        Parameters:
        ----------
        db_params (object): configuration data for accessing the database
        input_dir (string): directory (full path) to check for resting heart rate data files
        latest (Boolean): check for latest files only
        debug (Boolean): enable debug logging

        """
        logger.info("Processing rhr data")
        super().__init__(r'rhr_\d{4}-\d{2}-\d{2}\.json', input_dir=input_dir, latest=latest, debug=debug)
        self.garmin_db = GarminDb(db_params)
        self.conversions = {'statisticsStartDate': self._parse_date}

    def _process_json(self, json_data):
        rhr_list = json_data['allMetrics']['metricsMap']['WELLNESS_RESTING_HEART_RATE']
        if len(rhr_list) > 0:
            rhr = rhr_list[0].get('value')
            if rhr:
                point = {
                    'day': json_data['statisticsStartDate'].date(),
                    'resting_heart_rate': rhr
                }
                RestingHeartRate.insert_or_update(
                    self.garmin_db, point, ignore_none=True)
                return 1
        return 0


class GarminProfile(JsonFileProcessor):
    """Class for importing JSON formatted Garmin Connect profile data into a database."""

    def __init__(self, db_params, input_dir, debug):
        """
        Return an instance of GarminProfile.

        Parameters:
        ----------
        db_params (object): configuration data for accessing the database
        input_dir (string): directory (full path) to check for profile data files
        debug (Boolean): enable debug logging

        """
        logger.info("Processing profile data")
        super().__init__(r'profile\.json', input_dir=input_dir, latest=False, debug=debug)
        self.garmin_db = GarminDb(db_params)
        self.conversions = {'calendarDate': self._parse_date}

    def _process_json(self, json_data):
        measurement_system = fitfile.field_enums.DisplayMeasure.from_string(
            json_data['measurementSystem'])
        attributes = {
            'name': json_data['displayName'].replace('_', ' '),
            'time_zone': json_data['timeZone'],
            'measurement_system': str(measurement_system),
            'date_format': json_data['dateFormat']['formatKey']
        }
        for attribute_name, attribute_value in attributes.items():
            Attributes.set_newer(
                self.garmin_db, attribute_name, attribute_value)
        return len(attributes)


class GarminSummaryData(JsonFileProcessor):
    """Class for importing JSON formatted Garmin Connect daily summary data into a database."""

    def __init__(self, db_params, input_dir, latest, measurement_system, debug):
        """
        Return an instance of GarminSummaryData.

        Parameters:
        ----------
        db_params (dict): configuration data for accessing the database
        input_dir (string): directory (full path) to check for data files
        latest (Boolean): check for latest files only
        measurement_system (enum): which measurement system to use when importing the files
        debug (Boolean): enable debug logging

        """
        logger.info("Processing daily summary data")
        super().__init__(r'daily_summary_\d{4}-\d{2}-\d{2}\.json', input_dir=input_dir, latest=latest, debug=debug, recursive=True)
        self.input_dir = input_dir
        self.measurement_system = measurement_system
        self.garmin_db = GarminDb(db_params)
        self.conversions = {
            'calendarDate': self._parse_date,
            'moderateIntensityMinutes': fitfile.conversions.min_to_dt_time,
            'vigorousIntensityMinutes': fitfile.conversions.min_to_dt_time,
            'intensityMinutesGoal': fitfile.conversions.min_to_dt_time,
        }

    def _process_json(self, json_data):
        day = json_data['calendarDate'].date()
        distance = fitfile.Distance.from_meters(
            self._get_field(json_data, 'totalDistanceMeters', int))
        summary = {
            'day': day,
            'hr_min': self._get_field(json_data, 'minHeartRate', float),
            'hr_max': self._get_field(json_data, 'maxHeartRate', float),
            'rhr': self._get_field(json_data, 'restingHeartRate', float),
            'stress_avg': self._get_field(json_data, 'averageStressLevel', float),
            'step_goal': self._get_field(json_data, 'dailyStepGoal', int),
            'steps': self._get_field(json_data, 'totalSteps', int),
            'floors_goal': self._get_field(json_data, 'userFloorsAscendedGoal', float),
            'moderate_activity_time': json_data.get('moderateIntensityMinutes'),
            'vigorous_activity_time': json_data.get('vigorousIntensityMinutes'),
            'intensity_time_goal': json_data.get('intensityMinutesGoal'),
            'floors_up': self._get_field(json_data, 'floorsAscended', float),
            'floors_down': self._get_field(json_data, 'floorsDescended', float),
            'distance': distance.kms_or_miles(self.measurement_system),
            'calories_goal': self._get_field(json_data, 'netCalorieGoal', float),
            'calories_total': self._get_field(json_data, 'totalKilocalories', float),
            'calories_bmr': self._get_field(json_data, 'bmrKilocalories', float),
            'calories_active': self._get_field(json_data, 'activeKilocalories', float),
            'calories_consumed': self._get_field(json_data, 'consumedKilocalories', float),
            'spo2_avg': self._get_field(json_data, 'averageSpo2', float),
            'spo2_min': self._get_field(json_data, 'lowestSpo2', float),
            'rr_waking_avg': self._get_field(json_data, 'avgWakingRespirationValue', float),
            'rr_max': self._get_field(json_data, 'highestRespirationValue', float),
            'rr_min': self._get_field(json_data, 'lowestRespirationValue', float),
            'description': self._get_field(json_data, 'wellnessDescription'),
        }
        DailySummary.insert_or_update(
            self.garmin_db, summary, ignore_none=True)
        return 1


class GarminHydrationData(JsonFileProcessor):
    """Class for importing JSON formatted Garmin Connect daily summary data into a database."""

    def __init__(self, db_params, input_dir, latest, measurement_system, debug):
        """
        Return an instance of GarminHydrationData.

        Parameters:
        ----------
        db_params (object): configuration data for accessing the database
        input_dir (string): directory (full path) to check for data files
        latest (Boolean): check for latest files only
        measurement_system (enum): which measurement system to use when importing the files
        debug (Boolean): enable debug logging

        """
        logger.debug("Processing daily hydration data")
        super().__init__(r'hydration_\d{4}-\d{2}-\d{2}\.json', input_dir=input_dir, latest=latest, debug=debug, recursive=True)
        self.input_dir = input_dir
        self.measurement_system = measurement_system
        self.garmin_db = GarminDb(db_params)
        self.conversions = {
            'calendarDate': self._parse_date
        }

    def _process_json(self, json_data):
        hydration_intake = fitfile.Volume.from_milliliters(json_data['valueInML'])
        hydration_goal = fitfile.Volume.from_milliliters(json_data['baseGoalInML'])
        sweat_loss = fitfile.Volume.from_milliliters(json_data['sweatLossInML'])
        summary = {
            'day': json_data['calendarDate'].date(),
            'hydration_intake': hydration_intake.ml_or_oz(self.measurement_system, rounded=True),
            'hydration_goal': hydration_goal.ml_or_oz(self.measurement_system, rounded=True),
            'sweat_loss': sweat_loss.ml_or_oz(self.measurement_system, rounded=True)
        }
        root_logger.debug("Processing daily hydration data %r", summary)
        DailySummary.insert_or_update(
            self.garmin_db, summary, ignore_none=True)
        return 1



"""A building block for other tests."""

__author__ = "Tom Goetz"
__copyright__ = "Copyright Tom Goetz"
__license__ = "GPL"

import logging
import cProfile
import pstats


logger = logging.getLogger(__name__)


class TestDBBase(object):

    @classmethod
    def setUpClass(cls, db, table_dict, table_not_none_cols_dict={}):
        cls.db = db
        cls.table_dict = table_dict
        cls.table_not_none_cols_dict = table_not_none_cols_dict

    def profile_function(self, output_file_prefix, func, *args):
        pr = cProfile.Profile()
        pr.runcall(func, *args)
        ps_cum = pstats.Stats(pr, stream=open(output_file_prefix + '_cum.txt', 'w')).sort_stats('cumulative')
        ps_cum.print_stats()
        ps_tot = pstats.Stats(pr, stream=open(output_file_prefix + '_tot.txt', 'w')).sort_stats('tottime')
        ps_tot.print_stats()

    def check_not_none_cols(self, db, table_not_none_cols_dict):
        for table, not_none_cols_list in table_not_none_cols_dict.items():
            for not_none_col in not_none_cols_list:
                self.assertTrue(table.row_count(db, not_none_col, None) == 0, 'table %s col %s has None values' % (table, not_none_col))

    def check_db_tables_exists(self, db, table_dict, min_rows=1):
        for table_name, table in table_dict.items():
            logger.info("Checking %s exists", table_name)
            self.assertGreaterEqual(table.row_count(db), min_rows, 'table %s has no data' % table_name)

    def test_db_exists(self):
        logger.info("Checking DB %s exists", self.db.db_name)
        self.assertIsNotNone(self.db, 'DB %s doesnt exist' % self.db.db_name)

    def test_db_tables_exists(self):
        self.check_db_tables_exists(self.db, self.table_dict)

    def test_not_none_cols(self):
        self.check_not_none_cols(self.db, self.table_not_none_cols_dict)

    def check_col_type(self, db, table, col, type):
        for value in table.get_col_distinct(db, col):
            self.assertEqual(str(type(value)), value, f'table {table} col {col} has type {type} mismatch for {value}')


#!/usr/bin/env python3

"""A script that generated graphs for health data in a database."""

__author__ = "Tom Goetz"
__copyright__ = "Copyright Tom Goetz"
__license__ = "GPL"

import logging
import sys
import argparse

import matplotlib.pyplot as plt
import dateutil.parser

from garmindb import Graph, GarminConnectConfigManager, Statistics, format_version


logging.basicConfig(filename='graphs.log', filemode='w', level=logging.INFO)
logger = logging.getLogger(__file__)
logger.addHandler(logging.StreamHandler(stream=sys.stdout))
root_logger = logging.getLogger()

gc_config = GarminConnectConfigManager()


def main(argv):
    """Generate graphs based on commandline options."""
    def date_from_string(date_string):
        return dateutil.parser.parse(date_string).date()

    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--version", help="print the program's version", action='version', version=format_version(sys.argv[0]))
    parser.add_argument("-t", "--trace", help="Turn on debug tracing", type=int, default=0)
    parser.add_argument("-S", "--save", help="Save graphs to images files.", action="store_true", default=False)
    # stat types to operate on
    stats_group = parser.add_argument_group('Statistics', 'Graph statistics over a period of time')
    stats_group.add_argument("-A", "--all", help="Graph data for all enabled statistics.", action='store_const', dest='stats', const=gc_config.enabled_stats(), default=[])
    stats_group.add_argument("-m", "--monitoring", help="Graph monitoring data.", dest='stats', action='append_const', const=Statistics.monitoring)
    stats_group.add_argument("-r", "--hr", help="Graph heart rate data.", dest='stats', action='append_const', const=Statistics.rhr)
    stats_group.add_argument("-s", "--steps", help="Graph steps data.", dest='stats', action='append_const', const=Statistics.steps)
    stats_group.add_argument("-w", "--weight", help="Graph weight data.", dest='stats', action='append_const', const=Statistics.weight)
    stats_group.add_argument("-p", "--period", help="Graph period granularity.", dest='period', type=str, default=None, choices=['days', 'weeks', 'months'])
    daily_group = parser.add_argument_group('Daily')
    daily_group.add_argument("-d", "--day", help="Graph composite data for a single day.", type=date_from_string)
    modifiers_group = parser.add_argument_group('Modifiers')
    modifiers_group.add_argument("-l", "--latest", help="Graph the latest data.", dest='days', type=int, default=None)
    args = parser.parse_args()

    if args.trace > 0:
        root_logger.setLevel(logging.DEBUG)
    else:
        root_logger.setLevel(logging.INFO)

    graph = Graph(args.trace, args.save)

    if Statistics.rhr in args.stats:
        graph.graph_activity('hr', args.period, args.days)
        plt.show()

    if Statistics.itime in args.stats:
        graph.graph_activity('itime', args.period, args.days)
        plt.show()

    if Statistics.steps in args.stats:
        graph.graph_activity('steps', args.period, args.days)
        plt.show()

    if Statistics.weight in args.stats:
        graph.graph_activity('weight', args.period, args.days)
        plt.show()

    if args.day:
        graph.graph_date(args.day)
        plt.show()


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python3

"""
A script that imports and analyzes Garmin health device data into a database.

The data is either copied from a USB mounted Garmin device or downloaded from Garmin Connect.
"""

__author__ = "Tom Goetz"
__copyright__ = "Copyright Tom Goetz"
__license__ = "GPL"

import logging
import sys
import argparse
import datetime
import os
import tempfile

from garmindb import python_version_check, log_version, format_version
from garmindb.garmindb import GarminDb, Attributes, Sleep, Weight, RestingHeartRate, MonitoringDb, MonitoringHeartRate, ActivitiesDb, GarminSummaryDb
from garmindb.summarydb import SummaryDb

from garmindb import Download, Copy, Analyze
from garmindb import FitFileProcessor, ActivityFitFileProcessor, MonitoringFitFileProcessor
from garmindb import GarminProfile, GarminWeightData, GarminSummaryData, GarminMonitoringFitData, GarminSleepData, GarminRhrData, GarminSettingsFitData, GarminHydrationData
from garmindb import GarminJsonSummaryData, GarminJsonDetailsData, GarminTcxData, GarminActivitiesFitData
from garmindb import ActivityExporter

from garmindb import ConfigManager, GarminConnectConfigManager, PluginManager
from garmindb import Statistics
from garmindb import OpenWithBaseCamp, OpenWithGoogleEarth


logging.basicConfig(filename='garmin.log', filemode='w', level=logging.INFO)
logger = logging.getLogger(__file__)
logger.addHandler(logging.StreamHandler(stream=sys.stdout))
root_logger = logging.getLogger()

gc_config = GarminConnectConfigManager()
db_params_dict = ConfigManager.get_db_params()
plugin_manager = PluginManager(ConfigManager.get_or_create_plugins_dir(), db_params_dict)


stats_to_db_map = {
    Statistics.monitoring            : MonitoringDb,
    Statistics.steps                 : MonitoringDb,
    Statistics.itime                 : MonitoringDb,
    Statistics.sleep                 : GarminDb,
    Statistics.rhr                   : GarminDb,
    Statistics.weight                : GarminDb,
    Statistics.activities            : ActivitiesDb
}

summary_dbs = [GarminSummaryDb, SummaryDb]


def __get_date_and_days(db, latest, table, col, stat_name):
    if latest:
        last_ts = table.latest_time(db, col)
        if last_ts is None:
            date, days = gc_config.stat_start_date(stat_name)
            logger.info("Recent %s data not found, using: %s : %s", stat_name, date, days)
        else:
            # start from the day before the last day in the DB
            logger.info("Downloading latest %s data from: %s", stat_name, last_ts)
            date = last_ts.date() if isinstance(last_ts, datetime.datetime) else last_ts
            days = max((datetime.date.today() - date).days, 1)
    else:
        date, days = gc_config.stat_start_date(stat_name)
        days = min((datetime.date.today() - date).days, days)
        logger.info("Downloading all %s data from: %s [%d]", stat_name, date, days)
    if date is None or days is None:
        logger.error("Missing config: need %s_start_date and download_days. Edit GarminConnectConfig.py.", stat_name)
        sys.exit()
    return (date, days)


def copy_data(overwite, latest, stats):
    """Copy data from a mounted Garmin USB device to files."""
    logger.info("___Copying Data___")
    copy = Copy(gc_config.device_mount_dir())

    settings_dir = ConfigManager.get_or_create_fit_files_dir()
    root_logger.info("Copying settings to %s", settings_dir)
    copy.copy_settings(settings_dir)

    if Statistics.activities in stats:
        activities_dir = ConfigManager.get_or_create_activities_dir()
        root_logger.info("Copying activities to %s", activities_dir)
        copy.copy_activities(activities_dir, latest)

    if Statistics.monitoring in stats:
        monitoring_dir = ConfigManager.get_or_create_monitoring_dir(datetime.datetime.now().year)
        root_logger.info("Copying monitoring to %s", monitoring_dir)
        copy.copy_monitoring(monitoring_dir, latest)

    if Statistics.sleep in stats:
        monitoring_dir = ConfigManager.get_or_create_monitoring_dir(datetime.datetime.now().year)
        root_logger.info("Copying sleep to %s", monitoring_dir)
        copy.copy_sleep(monitoring_dir, latest)


def download_data(overwite, latest, stats):
    """Download selected activity types from Garmin Connect and save the data in files. Overwrite previously downloaded data if indicated."""
    logger.info("___Downloading %s Data___", 'Latest' if latest else 'All')

    download = Download()
    if not download.login():
        logger.error("Failed to login!")
        sys.exit()

    if Statistics.activities in stats:
        if latest:
            activity_count = gc_config.latest_activity_count()
        else:
            activity_count = gc_config.all_activity_count()
        activities_dir = ConfigManager.get_or_create_activities_dir()
        root_logger.info("Fetching %d activities to %s", activity_count, activities_dir)
        download.get_activity_types(activities_dir, overwite)
        download.get_activities(activities_dir, activity_count, overwite)

    if Statistics.monitoring in stats:
        date, days = __get_date_and_days(MonitoringDb(db_params_dict), latest, MonitoringHeartRate, MonitoringHeartRate.heart_rate, 'monitoring')
        if days > 0:
            root_logger.info("Date range to update: %s (%d) to %s", date, days, ConfigManager.get_monitoring_base_dir())
            download.get_daily_summaries(ConfigManager.get_or_create_monitoring_dir, date, days, overwite)
            download.get_hydration(ConfigManager.get_or_create_monitoring_dir, date, days, overwite)
            download.get_monitoring(ConfigManager.get_or_create_monitoring_dir, date, days)
            root_logger.info("Saved monitoring files for %s (%d) to %s for processing", date, days, ConfigManager.get_monitoring_base_dir())

    if Statistics.sleep in stats:
        date, days = __get_date_and_days(GarminDb(db_params_dict), latest, Sleep, Sleep.total_sleep, 'sleep')
        if days > 0:
            sleep_dir = ConfigManager.get_or_create_sleep_dir()
            root_logger.info("Date range to update: %s (%d) to %s", date, days, sleep_dir)
            download.get_sleep(sleep_dir, date, days, overwite)
            root_logger.info("Saved sleep files for %s (%d) to %s for processing", date, days, sleep_dir)

    if Statistics.weight in stats:
        date, days = __get_date_and_days(GarminDb(db_params_dict), latest, Weight, Weight.weight, 'weight')
        if days > 0:
            weight_dir = ConfigManager.get_or_create_weight_dir()
            root_logger.info("Date range to update: %s (%d) to %s", date, days, weight_dir)
            download.get_weight(weight_dir, date, days, overwite)
            root_logger.info("Saved weight files for %s (%d) to %s for processing", date, days, weight_dir)

    if Statistics.rhr in stats:
        date, days = __get_date_and_days(GarminDb(db_params_dict), latest, RestingHeartRate, RestingHeartRate.resting_heart_rate, 'rhr')
        if days > 0:
            rhr_dir = ConfigManager.get_or_create_rhr_dir()
            root_logger.info("Date range to update: %s (%d) to %s", date, days, rhr_dir)
            download.get_rhr(rhr_dir, date, days, overwite)
            root_logger.info("Saved rhr files for %s (%d) to %s for processing", date, days, rhr_dir)


def import_data(debug, latest, stats):
    """Import previously downloaded Garmin data into the database."""
    logger.info("___Importing %s Data___", 'Latest' if latest else 'All')

    # Import the user profile and/or settings FIT file first so that we can get the measurement system and some other things sorted out first.
    fit_files_dir = ConfigManager.get_or_create_fit_files_dir()
    gp = GarminProfile(db_params_dict, fit_files_dir, debug)
    if gp.file_count() > 0:
        gp.process()

    gsfd = GarminSettingsFitData(fit_files_dir, debug)
    if gsfd.file_count() > 0:
        gsfd.process_files(FitFileProcessor(db_params_dict, plugin_manager, debug))

    gdb = GarminDb(db_params_dict)
    measurement_system = Attributes.measurements_type(gdb)

    if Statistics.weight in stats:
        weight_dir = ConfigManager.get_or_create_weight_dir()
        gwd = GarminWeightData(db_params_dict, weight_dir, latest, measurement_system, debug)
        if gwd.file_count() > 0:
            gwd.process()

    if Statistics.monitoring in stats:
        monitoring_dir = ConfigManager.get_or_create_monitoring_base_dir()
        gsd = GarminSummaryData(db_params_dict, monitoring_dir, latest, measurement_system, debug)
        if gsd.file_count() > 0:
            gsd.process()

        ghd = GarminHydrationData(db_params_dict, monitoring_dir, latest, measurement_system, debug)
        if ghd.file_count() > 0:
            ghd.process()

        gfd = GarminMonitoringFitData(monitoring_dir, latest, measurement_system, debug)
        if gfd.file_count() > 0:
            gfd.process_files(MonitoringFitFileProcessor(db_params_dict, plugin_manager, debug))

    if Statistics.sleep in stats:
        sleep_dir = ConfigManager.get_or_create_sleep_dir()
        gsd = GarminSleepData(db_params_dict, sleep_dir, latest, debug)
        if gsd.file_count() > 0:
            gsd.process()

    if Statistics.rhr in stats:
        rhr_dir = ConfigManager.get_or_create_rhr_dir()
        grhrd = GarminRhrData(db_params_dict, rhr_dir, latest, debug)
        if grhrd.file_count() > 0:
            grhrd.process()

    if Statistics.activities in stats:
        activities_dir = ConfigManager.get_or_create_activities_dir()
        # Tcx fields are less precise than the JSON files, so load Tcx first and overwrite with better JSON values.
        gtd = GarminTcxData(activities_dir, latest, measurement_system, debug)
        if gtd.file_count() > 0:
            gtd.process_files(db_params_dict)

        gjsd = GarminJsonSummaryData(db_params_dict, activities_dir, latest, measurement_system, debug)
        if gjsd.file_count() > 0:
            gjsd.process()

        gdjd = GarminJsonDetailsData(db_params_dict, activities_dir, latest, measurement_system, debug)
        if gdjd.file_count() > 0:
            gdjd.process()

        gfd = GarminActivitiesFitData(activities_dir, latest, measurement_system, debug)
        if gfd.file_count() > 0:
            gfd.process_files(ActivityFitFileProcessor(db_params_dict, plugin_manager, debug))


def analyze_data(debug):
    """Analyze the downloaded and imported Garmin data and create summary tables."""
    logger.info("___Analyzing Data___")
    analyze = Analyze(db_params_dict, debug - 1)
    analyze.get_stats()
    analyze.summary()
    analyze.create_dynamic_views()


def delete_dbs(delete_db_list=[]):
    """Delete selected, or all if none selected GarminDb, database files."""
    if len(delete_db_list) == 0:
        delete_db_list = [GarminDb, MonitoringDb, ActivitiesDb, GarminSummaryDb, SummaryDb]
    for db in delete_db_list:
        db.delete_db(db_params_dict)


def export_activity(debug, directory, export_activity_id):
    """Export an activity given its database id."""
    garmin_db = GarminDb(db_params_dict)
    measurement_system = Attributes.measurements_type(garmin_db)
    ae = ActivityExporter(directory, export_activity_id, measurement_system, debug)
    ae.process(db_params_dict)
    return ae.write('activity_%s.tcx' % export_activity_id)


def basecamp_activity(debug, export_activity_id):
    """Export an activity given its database id."""
    file_with_path = export_activity(debug, tempfile.mkdtemp(), export_activity_id)
    logger.info("Opening activity %d (%s) in BaseCamp", export_activity_id, file_with_path)
    OpenWithBaseCamp.open(file_with_path)


def google_earth_activity(debug, export_activity_id):
    """Export an activity given its database id."""
    file_with_path = export_activity(debug, tempfile.mkdtemp(), export_activity_id)
    logger.info("Opening activity %d (%s) in GoogleEarth", export_activity_id, file_with_path)
    OpenWithGoogleEarth.open(file_with_path)


def main(argv):
    """Manage Garmin device data."""
    python_version_check(sys.argv[0])

    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--version", help="print the program's version", action='version', version=format_version(sys.argv[0]))
    parser.add_argument("-t", "--trace", help="Turn on debug tracing", type=int, default=0)
    modes_group = parser.add_argument_group('Modes')
    modes_group.add_argument("-d", "--download", help="Download data from Garmin Connect for the chosen stats.", dest='download_data', action="store_true", default=False)
    modes_group.add_argument("-c", "--copy", help="copy data from a connected device", dest='copy_data', action="store_true", default=False)
    modes_group.add_argument("-i", "--import", help="Import data for the chosen stats", dest='import_data', action="store_true", default=False)
    modes_group.add_argument("--analyze", help="Analyze data in the db and create summary and derived tables.", dest='analyze_data', action="store_true", default=False)
    modes_group.add_argument("--delete_db", help="Delete Garmin DB db files for the selected activities.", action="store_true", default=False)
    modes_group.add_argument("-e", "--export-activity", help="Export an activity to a TCX file based on the activity\'s id", type=int)
    modes_group.add_argument("-b", "--basecamp-activity", help="Export an activity to Garmin BaseCamp", type=int)
    modes_group.add_argument("-g", "--google-earth-activity", help="Export an activity to Google Earth", type=int)
    # stat types to operate on
    stats_group = parser.add_argument_group('Statistics')
    stats_group.add_argument("-A", "--all", help="Download and/or import data for all enabled stats.", action='store_const', dest='stats',
                             const=gc_config.enabled_stats(), default=[])
    stats_group.add_argument("-a", "--activities", help="Download and/or import activities data.", dest='stats', action='append_const', const=Statistics.activities)
    stats_group.add_argument("-m", "--monitoring", help="Download and/or import monitoring data.", dest='stats', action='append_const', const=Statistics.monitoring)
    stats_group.add_argument("-r", "--rhr", help="Download and/or import resting heart rate data.", dest='stats', action='append_const', const=Statistics.rhr)
    stats_group.add_argument("-s", "--sleep", help="Download and/or import sleep data.", dest='stats', action='append_const', const=Statistics.sleep)
    stats_group.add_argument("-w", "--weight", help="Download and/or import weight data.", dest='stats', action='append_const', const=Statistics.weight)
    modifiers_group = parser.add_argument_group('Modifiers')
    modifiers_group.add_argument("-l", "--latest", help="Only download and/or import the latest data.", action="store_true", default=False)
    modifiers_group.add_argument("-o", "--overwrite", help="Overwite existing files when downloading. The default is to only download missing files.",
                                 action="store_true", default=False)
    args = parser.parse_args()

    log_version(sys.argv[0])

    if args.trace > 0:
        root_logger.setLevel(logging.DEBUG)
    else:
        root_logger.setLevel(logging.INFO)

    root_logger.info("Enabled statistics: %r", args.stats)

    if args.delete_db:
        delete_dbs([stats_to_db_map[stat] for stat in args.stats] + summary_dbs)
        sys.exit()

    if args.copy_data:
        copy_data(args.overwrite, args.latest, args.stats)

    if args.download_data:
        download_data(args.overwrite, args.latest, args.stats)

    if args.import_data:
        import_data(args.trace, args.latest, args.stats)

    if args.analyze_data:
        analyze_data(args.trace)

    if args.export_activity:
        export_activity(args.trace, os.getcwd(), args.export_activity)

    if args.basecamp_activity:
        basecamp_activity(args.trace, args.basecamp_activity)

    if args.google_earth_activity:
        google_earth_activity(args.trace, args.google_earth_activity)


if __name__ == "__main__":
    main(sys.argv[1:])
